from urllib.request import Request
from urllib.request import urlopen
from urllib.error import URLError
import json
import py_openthesaurus.log as log
import re


class OpenThesaurus(object):

    def __init__(self, word):
        self.word = word
        self.umlaut_dictionary = {ord(u"ü"): u"ue", ord(u"Ü"): u"Ue", ord(u"ß"): u"ss", ord(u"ä"): u"ae",
                                  ord(u"Ä"): u"Ae",
                                  ord(u"ö"): u"oe",
                                  ord(u"Ö"): u"Oe"}
        self.url = "http://www.openthesaurus.de/synonyme/search?format=application/json&q=%s"

    def is_entry_word_valid(self):

        if self.word is None:
            return False

        regex = r"[^a-zA-Z]+"
        word = re.sub(regex, "", self.word)
        return word is not ""

    def get_synonyms(self, form="short"):

        logger = log.setup_custom_logger(__name__)

        try:

            if self.is_entry_word_valid():
                url = self.url % self.word.translate(self.umlaut_dictionary)
                request = Request(url)
                with urlopen(request) as response:
                    return self._get_synonyms_from_response(response, form)
            else:
                logger.warn('Please provide a valid (non empty, non null) input word!')
                return []
        except URLError:
            logger.error(
                "Please check your internet connection! If your internet connection is ok, Open Thesaurus URL or REST "
                "API changed!")
            return []
        except RuntimeError as runtime_error:
            logger.error(repr(runtime_error))
            return []
        except Exception as exception:
            logger.error(repr(exception))
            return []

    def _get_synonyms_from_response(self, response, form):
        synonyms = []
        json_response = json.loads(response.read())

        for category in json_response.get("synsets"):
            synonyms += self._get_synonyms_from_category(category=category, form=form)

        return synonyms

    def _get_synonyms_from_category(self, category, form):

        long_form_regex = r"[()]"
        short_form_regex = r"[\(].*?[\)]"

        if form == "long":
            return [re.sub(long_form_regex, "", synonym.get("term")).strip() for synonym in category.get("terms")]
        elif form == "short":
            return [re.sub(short_form_regex, "", synonym.get("term")).strip() for synonym in category.get("terms")]
        else:
            raise RuntimeError("Form is not valid! Please choose form to be either: long or short")



from setuptools import setup, find_packages

with open("README.md", encoding="utf-8") as file:
    description = file.read()

setup(name="py_openthesaurus",
      version="1.0.2",
      description="Python wrapper for obtaining synonyms in the German language from OpenThesaurus",
      long_description=description,
      long_description_content_type="text/markdown",
      classifiers=[
          "Development Status :: 5 - Production/Stable",
          "Intended Audience :: Developers",
          "Intended Audience :: Education",
          "Intended Audience :: Information Technology",
          "Intended Audience :: Science/Research",
          "Programming Language :: Python :: 3",
          "License :: OSI Approved :: MIT License",
          "Topic :: Text Processing :: General"],
      url="https://github.com/Aid91/py_openthesaurus",
      keywords=["Natural Language Processing", "NLP", "German"],
      author="Aid Ahmetovic",
      author_email="aidahmetovic91@gmail.com",
      license="MIT",
      packages=find_packages(exclude=["tests*"]),
      entry_points={"console_scripts": ["py_openthesaurus=py_openthesaurus.main:main"]},
      python_requires=">=3.5")



from py_openthesaurus import OpenThesaurus
import py_openthesaurus.log as log
import argparse


def main():
    logger = log.setup_custom_logger(__name__)

    parser = argparse.ArgumentParser(
        description='Get synonyms of German words from www.openthesaurus.de')

    parser.add_argument('--form', required=False, action='store', choices=['long', 'short'], type=str, default='short',
                        help="Defaults to form=short which means that short versions of synonyms will be returned, "
                             "without nach/zu prefixes/suffixes."
                             "On the other hand, form=long returns the full versions of synonyms including nach/zu, "
                             "sich prefixes/suffixes")

    required = parser.add_argument_group('required arguments')

    required.add_argument('--word', type=str, action='store', required=True,
                          help="A word from which synonyms will be obtained")

    args = parser.parse_args()

    open_thesaurus = OpenThesaurus(word=args.word)

    synonyms = open_thesaurus.get_synonyms(form=args.form)

    logger.info(synonyms)


if __name__ == '__main__':
    main()
